// Copyright 2023 Espressif Systems (Shanghai) PTE LTD
// All rights reserved.

#pragma once

#include "stdint.h"

/* Error code */
typedef enum {
    JPEG_ERR_OK           = 0,  /* Succeeded */
    JPEG_ERR_FAIL         = -1, /* Device error or wrong termination of input stream */
    JPEG_ERR_MEM          = -2, /* Insufficient memory pool for the image */
    JPEG_ERR_NO_MORE_DATA = -3, /* Input data is not enough */
    JPEG_ERR_PAR          = -4, /* Parameter error */
    JPEG_ERR_FMT1         = -5, /* Data format error (may be damaged data) */
    JPEG_ERR_FMT2         = -6, /* Right format but not supported */
    JPEG_ERR_FMT3         = -7  /* Not supported JPEG standard */
} jpeg_error_t;

/* Jpeg color space type.
 * raw data ---> encoder  --->  encoded data ---> transmit  ---> decoder ---> raw data
 */
typedef enum {
    JPEG_COLOR_SPACE_GRAY       = 0, /*!< Grayscale. Encoder supported. Decoder un-supported.*/
    JPEG_COLOR_SPACE_RGB888     = 1, /*!< RGB888. Encoder supported.  Decoder supported. */
    JPEG_COLOR_SPACE_RGBA       = 2, /*!< RGBA. Encoder supported.  Decoder un-supported.*/
    JPEG_COLOR_SPACE_YCbYCr     = 3, /*!< Y422. Data order is YCbYCr. Encoder supported.  Decoder un-supported.*/
    JPEG_COLOR_SPACE_YCbY2YCrY2 = 4, /*!< Y420. Encoder supported.  Decoder un-supported.*/
    JPEG_COLOR_SPACE_RGB565_BE  = 5, /*!< RGB565. The data is big end. Encoder un-supported.  Decoder supported. */
    JPEG_COLOR_SPACE_RGB565_LE  = 6, /*!< RGB565. The data is little end. Encoder un-supported.  Decoder supported.*/
    JPEG_COLOR_SPACE_CbYCrY     = 7, /*!< Y422. Data order is CbYCrY. Encoder un-supported.  Decoder supported.*/
} jpeg_color_space_t;

/* JPEG chroma subsampling factors */
typedef enum {
    JPEG_SUB_SAMPLE_Y      = 0, /*!< Grayscale */
    JPEG_SUB_SAMPLE_YUV444 = 1, /*!< Data order:YUV 1x1x1   3 pixel per MCU  Y444*/
    JPEG_SUB_SAMPLE_YUV422 = 2, /*!< Data order:YUYV 2x1x1  2 pixel per MCU  Y422*/
    JPEG_SUB_SAMPLE_YUV420 = 3  /*!< Data order:YUY2YVY2 4x1x1  1.25 pixel per MCU  Y420)*/
} jpeg_subsampling_t;

typedef enum {
    JPEG_ROTATE_0D   = 0, /*!< Source image rotates clockwise 0 degree before encoding */
    JPEG_ROTATE_90D  = 1, /*!< Source image rotates clockwise 90 degree before encoding */
    JPEG_ROTATE_180D = 2, /*!< Source image rotates clockwise 180 degree before encoding  */
    JPEG_ROTATE_270D = 3, /*!< Source image rotates clockwise 270 degree before encoding  */
} jpeg_rotate_t;

typedef struct {
    int width;
    int height;
} jpeg_resolution_t;

/**
 * @brief      Convert YUV to RGB
 *
 * @param[in]      sub_sample    Chroma subsampling factors. Grayscale is un-supported. The others are supported.
 * @param[in]      out_type      Output data color space type. RGB888 and RGB565 are supported. The others are un-supported.
 * @param[in]      yuv_image     Input YUV image data. The buffer must be aligned 16 byte.
 * @param[in]      width         Image width
 * @param[in]      height        Image height
 * @param[out]     rgb_image     Output RGB image data
 *
 * @return     jpeg_error_t
 *             - JPEG_ERR_OK: On success
 *             - Others: Error occurs
 */
jpeg_error_t jpeg_yuv2rgb(jpeg_subsampling_t sub_sample, jpeg_color_space_t out_type, uint8_t *yuv_image, int width, int height, uint8_t *rgb_image);

/**
 * @brief      Allocated buffer. And the buffer address will be aligned.
 *
 * @param[in]      size    Allocate buffer size.
 * @param[in]      aligned Aligned byte
 *
 * @return     Positive:   Allocate buffer address
 *             NULL: Failed
 */
void *jpeg_malloc_align(int size, int aligned);

/**
 * @brief      Free buffer. The buffer address come from `jpeg_malloc_align`
 *
 * @param[in]      data    The buffer address.
 *
 */
void jpeg_free_align(void *data);

/**
 * Example usage:
 * @code{c}
 *
 * add
 *
 * @endcode
 */