// Copyright 2023 Espressif Systems (Shanghai) PTE LTD
// All rights reserved.

#pragma once

#ifdef __cplusplus
extern "C" {
#endif

#include "esp_jpeg_common.h"

#define DEFAULT_JPEG_DEC_CONFIG() {              \
    .output_type = JPEG_COLOR_SPACE_RGB565_LE,   \
    .scale       = { .width = 0, .height = 0 },  \
    .clipper     = { .width = 0, .height = 0 },  \
    .rotate      = JPEG_ROTATE_0D,               \
    .block_enable = 0,                           \
}

#define JPEG_DEC_MAX_MARKER_CHECK_LEN (1024)

typedef void *jpeg_dec_handle_t;

/* Jpeg dec user need to config */
typedef struct {
    jpeg_color_space_t output_type; /* Output type. Support RGB888 RGB565_BE RGB565_LE CbYCrY, see jpeg_color_space_t enumeration */
    jpeg_resolution_t  scale;       /* Resolution of scale. Configuring to 0 means no scale */
    jpeg_resolution_t  clipper;     /* Resolution of clipper. Configuring to 0 means no clipper */
    jpeg_rotate_t      rotate;      /* Support 0 90 180 270 degree clockwise rotation.
                                       Under width % 8 == 0. height % 8 = 0 conditions, rotation enabled. Otherwise unsupported */
    int block_enable;               /* True: enable block mode; If enable this mode, the width and height must be times of 8. And rotate is JPEG_ROTATE_0D.
                                       False: disable block mode */
} jpeg_dec_config_t;

/* Jpeg dec out info */
typedef struct {
    uint16_t width;        /* Number of pixels in the horizontal direction */
    uint16_t height;       /* Number of pixels in the vertical direction */
    uint8_t  x_factory[3]; /* Size factory in the x direction*/
    uint8_t  y_factory[3]; /* Size factory in the y direction*/
} jpeg_dec_header_info_t;

/* Jpeg dec io control */
typedef struct {
    unsigned char *inbuf;         /* The input buffer pointer */
    int            inbuf_len;     /* The number of the input buffer */
    int            inbuf_remain;  /* Not used number of the in buffer */
    unsigned char *outbuf;        /* The decoded data is placed.The buffer must be aligned 16 byte */
    uint16_t       output_line;   /* Row index of next line to be read, only take effect when block_enable is set */
    uint16_t       output_height; /* Total rows of image, only take effect when block_enable is set */
    uint16_t       cur_line;      /* Current output line of block, only take effect when block_enable is set */
    int            cur_len;       /* Current output bytes of block, only take effect when block_enable is set */
} jpeg_dec_io_t;

/**
 * @brief      Create a Jpeg decode handle, set user config info to decode handle
 *
 * @param[in]      config        The configuration information
 *
 * @return     other values: The JPEG decoder handle
 *             NULL: failed
 */
jpeg_dec_handle_t *jpeg_dec_open(jpeg_dec_config_t *config);

/**
 * @brief      Parse picture data header, and out put info to user
 *
 * @param[in]      jpeg_dec        jpeg decoder handle
 *
 * @param[in]      io              struct of jpeg_dec_io_t
 *
 * @param[out]     out_info        output info struct to user
 *
 * @return     jpeg_error_t
 *             - JPEG_ERR_OK: on success
 *             - Others: error occurs
 */
jpeg_error_t jpeg_dec_parse_header(jpeg_dec_handle_t *jpeg_dec, jpeg_dec_io_t *io, jpeg_dec_header_info_t *out_info);

/**
 * @brief      Decode one Jpeg picture
 *
 * @param[in]      jpeg_dec    jpeg decoder handle
 *
 * @param[in]      io          struct of jpeg_dec_io_t
 *
 * @return     jpeg_error_t
 *             - JPEG_ERR_OK: on success
 *             - Others: error occurs
 */
jpeg_error_t jpeg_dec_process(jpeg_dec_handle_t *jpeg_dec, jpeg_dec_io_t *io);

/**
 * @brief      Deinitialize Jpeg decode handle
 *
 * @param[in]      jpeg_dec    jpeg decoder handle
 *
 * @return     jpeg_error_t
 *             - JPEG_ERR_OK: on success
 *             - Others: error occurs
 */
jpeg_error_t jpeg_dec_close(jpeg_dec_handle_t *jpeg_dec);

#ifdef __cplusplus
}
#endif